#! /bin/sh

set -e

gemfiles_req () {
  local pkgprefix='rubygem('
  local pkgsuffix=')'
  local cmdprefix='Requires:       '
  local -A ignored_groups
  local cmd arg1 arg2 ignore op ver maxver p
  while [ "$#" -gt 0 ]; do
    case "$1" in
    -b) cmdprefix='BuildRequires:  '; shift;;
    -x) shift; ignored_groups[":$1"]=1; shift;;
    -x*) ignored_groups[":${1:2}"]=1; shift;;
    -*) shift;;
    *) break;;
    esac
  done
  for file do
    ignore=false
    cat "$file" | sed -e 's/#.*$//' -e '/^ *$/d' |
    while read cmd arg1 arg2; do
      case "$cmd" in
      group)
        if [ "${ignored_groups[${arg1}]:-0}" -ne 0 ]; then
          ignore=true
        else
          ignore=false
        fi
        ;;
      gem)
        if ! $ignore; then
          case "$arg1" in
          *,) arg1="${arg1:0:${#arg1}-1}" ;;
          esac
          case "$arg1" in
          \'*\'|\"*\") arg1="${arg1:1:${#arg1}-2}" ;;
          esac
          arg2="${arg2%%,*}"
          case "$arg2" in
          \'*\'|\"*\") arg2="${arg2:1:${#arg2}-2}" ;;
          esac
          maxver=false
          case "$arg2" in
          \>=*) op='>='; ver="${arg2:2}";;
          \>*) op='>'; ver="${arg2:1}";;
          \<=*) op='<='; ver="${arg2:2}";;
          \<*) op='<'; ver="${arg2:1}";;
          \~\>*) op='>='; ver="${arg2:2}"; maxver=true;;
          *) op=; ver=;;
          esac
          while :; do case "$ver" in \ *) ver="${ver:1}";; *) break; esac; done
          if [ -n "$ver" ]; then
            echo "${cmdprefix}${pkgprefix}${arg1}${pkgsuffix} ${op} ${ver}"
          else
            echo "${cmdprefix}${pkgprefix}${arg1}${pkgsuffix}"
          fi
          if $maxver; then
            ver="${ver%.*}"
            case "$ver" in
            *.*) p="${ver%.*}."; ver="${ver##*.}";;
            *) p=;;
            esac
            let ver="${ver}+1"
            echo "${cmdprefix}${pkgprefix}${arg1}${pkgsuffix} < ${p}${ver}"
          fi
        fi
        ;;
      end)
        ignore=false
        ;;
      *)
        ;;
      esac
    done
  done
}

specfile=`ls -1 *.spec`

rm -rf open-build-service
git clone --depth=1000 git://github.com/openSUSE/open-build-service.git --single-branch --branch 2.5
cd open-build-service
commit=`git show | grep ^commit | head -n 1 | sed -e 's,commit ,,; s,\(........\).*,\1,'`
descr=`git describe --long --tags | sed 's/-.*$//'`
git checkout "$descr"
cd ..

if `ls -1 obs-server*.xz 2>/dev/null | grep -q $descr.tar`; then
  echo "leaving old sources"
else
  cp -al open-build-service obs-server-$descr
  rm -f obs-server*xz
  tar -cJf obs-server-$descr.tar.xz --exclude=.git --exclude=.gitignore obs-server-$descr
  sed -i -e "s,^\(Version: *\).*\$,\1$descr," $specfile
  rm -rf obs-server-$descr
  osc addremove || :
fi

egrep '^Patch[0-9]*:' $specfile |
sed 's/^Patch\([0-9]*\): *\([^ ]*\).*$/\1 \2/'|
while read num patchfile; do
  patchopts="`grep \"^%patch$num \" $specfile | sed \"s/%patch$num *//\"`"
  echo patch -d open-build-service $patchopts '<' "$patchfile"
  patch -d open-build-service $patchopts < "$patchfile"
done

tdir=`mktemp -d`
pushd open-build-service/src/api
gemfiles_req -b Gemfile | sort -u > $tdir/api-br
gemfiles_req -xtest -xassets -xdevelopment Gemfile | sort -u > $tdir/api-r
popd

{ sed -n -e '1,/OBS_SERVER_BUILDREQUIRES_BEGIN/p' $specfile
  cat $tdir/api-br
  sed -n -e '/OBS_SERVER_BUILDREQUIRES_END/,$p' $specfile
} > $specfile.tmp && mv $specfile.tmp $specfile

{ sed -n -e '1,/OBS_API_REQUIRES_BEGIN/p' $specfile
  cat $tdir/api-r
  sed -n -e '/OBS_API_REQUIRES_END/,$p' $specfile
} > $specfile.tmp && mv $specfile.tmp $specfile

rm -rf open-build-service $tdir
osc service localrun format_spec_file || :
